@@ -46,8 +46,8 @@ module Sass
         @actively_checking = Set.new
 
         # Entries in the following instance-level caches are never explicitly expired.
-        # Instead they are supposed to automaticaly go out of scope when a series of staleness checks
-        # (this instance of StalenessChecker was created for) is finished.
+        # Instead they are supposed to automatically go out of scope when a series of staleness
+        # checks (this instance of StalenessChecker was created for) is finished.
         @mtimes, @dependencies_stale, @parse_trees = {}, {}, {}
         @options = Sass::Engine.normalize_options(options)
       end
@@ -156,7 +156,7 @@ module Sass
       end
 
       def dependency_updated?(css_mtime)
-        Proc.new do |uri, importer|
+        proc do |uri, importer|
           next true if @actively_checking.include?(uri)
           begin
             @actively_checking << uri
