@@ -1,4 +1,5 @@
 require 'sass/script/lexer'
+require 'sass/script/css_variable_warning'
 
 module Sass
   module Script
@@ -24,11 +25,17 @@ module Sass
       #   Used for error reporting and sourcemap building
       # @param offset [Fixnum] The character (not byte) offset where the script starts in the line.
       #   Used for error reporting and sourcemap building
-      # @param options [{Symbol => Object}] An options hash;
-      #   see {file:SASS_REFERENCE.md#sass_options the Sass options documentation}
+      # @param options [{Symbol => Object}] An options hash; see
+      #   {file:SASS_REFERENCE.md#sass_options the Sass options documentation}.
+      #   This supports an additional `:allow_extra_text` option that controls
+      #   whether the parser throws an error when extra text is encountered
+      #   after the parsed construct.
       def initialize(str, line, offset, options = {})
         @options = options
+        @allow_extra_text = options.delete(:allow_extra_text)
         @lexer = lexer_class.new(str, line, offset, options)
+        @stop_at = nil
+        @css_variable_warning = nil
       end
 
       # Parses a SassScript expression within an interpolated segment (`#{}`).
@@ -46,7 +53,7 @@ module Sass
         expr = assert_expr :expr
         assert_tok :end_interpolation
         expr = Sass::Script::Tree::Interpolation.new(
-          nil, expr, nil, !:wb, !:wa, :warn_for_color => warn_for_color)
+          nil, expr, nil, false, false, :warn_for_color => warn_for_color)
         check_for_interpolation expr
         expr.options = @options
         node(expr, start_pos)
@@ -57,13 +64,23 @@ module Sass
 
       # Parses a SassScript expression.
       #
+      # @param css_variable [Boolean] Whether this is the value of a CSS variable.
       # @return [Script::Tree::Node] The root node of the parse tree
       # @raise [Sass::SyntaxError] if the expression isn't valid SassScript
-      def parse
+      def parse(css_variable = false)
+        if css_variable
+          @css_variable_warning = CssVariableWarning.new
+        end
+
         expr = assert_expr :expr
         assert_done
         expr.options = @options
         check_for_interpolation expr
+
+        if css_variable
+          @css_variable_warning.value = expr
+        end
+
         expr
       rescue Sass::SyntaxError => e
         e.modify_backtrace :line => @lexer.line, :filename => @options[:filename]
@@ -215,12 +232,12 @@ module Sass
 
       # Parses a SassScript expression.
       #
-      # @overload parse(str, line, offset, filename = nil)
       # @return [Script::Tree::Node] The root node of the parse tree
       # @see Parser#initialize
       # @see Parser#parse
-      def self.parse(*args)
-        new(*args).parse
+      def self.parse(value, line, offset, options = {})
+        css_variable = options.delete :css_variable
+        new(value, line, offset, options).parse(css_variable)
       end
 
       PRECEDENCE = [
@@ -233,6 +250,8 @@ module Sass
 
       ASSOCIATIVE = [:plus, :times]
 
+      VALID_CSS_OPS = [:comma, :single_eq, :space, :div]
+
       class << self
         # Returns an integer representing the precedence
         # of the given operator.
@@ -272,6 +291,10 @@ module Sass
                   return other_interp
                 end
 
+                if @css_variable_warning && !VALID_CSS_OPS.include?(tok.type)
+                  @css_variable_warning.warn!
+                end
+
                 e = node(Tree::Operation.new(e, assert_expr(#{sub.inspect}), tok.type),
                          e.source_range.start_pos)
               end
@@ -287,6 +310,8 @@ RUBY
               interp = try_op_before_interp(tok)
               return interp if interp
               start_pos = source_position
+
+              @css_variable_warning.warn! if @css_variable_warning
               node(Tree::UnaryOperation.new(assert_expr(:unary_#{op}), :#{op}), start_pos)
             end
 RUBY
@@ -313,6 +338,7 @@ RUBY
         return list e, start_pos unless @lexer.peek && @lexer.peek.type == :colon
 
         pair = map_pair(e)
+        @css_variable_warning.warn! if @css_variable_warning
         map = node(Sass::Script::Tree::MapLiteral.new([pair]), start_pos)
         while try_tok(:comma)
           pair = map_pair
@@ -373,7 +399,7 @@ RUBY
 
         interp = node(
           Script::Tree::Interpolation.new(
-            prev, str, nil, wb, !:wa, :originally_text => true, :deprecation => deprecation),
+            prev, str, nil, wb, false, :originally_text => true, :deprecation => deprecation),
           (prev || str).source_range.start_pos)
         interpolation(interp)
       end
@@ -399,7 +425,7 @@ RUBY
           end
         interp = node(
           Script::Tree::Interpolation.new(
-            prev, str, assert_expr(name), !:wb, wa,
+            prev, str, assert_expr(name), false, wa,
             :originally_text => true, :deprecation => deprecation),
           (prev || str).source_range.start_pos)
         interp
@@ -410,7 +436,7 @@ RUBY
         while (interp = try_tok(:begin_interpolation))
           wb = @lexer.whitespace?(interp)
           char_before = @lexer.char(interp.pos - 1)
-          mid = assert_expr :expr
+          mid = without_css_variable_warning {assert_expr :expr}
           assert_tok :end_interpolation
           wa = @lexer.whitespace?
           char_after = @lexer.char
@@ -445,18 +471,24 @@ RUBY
       #   `expr`).
       # @return [Boolean]
       def is_unsafe_before?(expr, char_before)
-        # If the previous expression is an identifier or number, it's safe
-        # unless it was wrapped in parentheses.
-        if expr.is_a?(Script::Tree::Literal) &&
-           (expr.value.is_a?(Script::Value::Number) ||
-            (expr.value.is_a?(Script::Value::String) && expr.value.type == :identifier))
-          return char_before == ')'
-        end
+        return char_before == ')' if is_safe_value?(expr)
 
         # Otherwise, it's only safe if it was another interpolation.
         !expr.is_a?(Script::Tree::Interpolation)
       end
 
+      # Returns whether `expr` is safe as the value immediately before an
+      # interpolation.
+      #
+      # It's safe as long as the previous expression is an identifier or number,
+      # or a list whose last element is also safe.
+      def is_safe_value?(expr)
+        return is_safe_value?(expr.elements.last) if expr.is_a?(Script::Tree::ListLiteral)
+        return false unless expr.is_a?(Script::Tree::Literal)
+        expr.value.is_a?(Script::Value::Number) ||
+          (expr.value.is_a?(Script::Value::String) && expr.value.type == :identifier)
+      end
+
       def space
         start_pos = source_position
         e = or_expr
@@ -599,11 +631,11 @@ RUBY
         return paren unless first
         str = literal_node(first.value, first.source_range)
         return str unless try_tok(:string_interpolation)
-        mid = assert_expr :expr
+        mid = without_css_variable_warning {assert_expr :expr}
         assert_tok :end_interpolation
         last = assert_expr(:special_fun)
         node(
-          Tree::Interpolation.new(str, mid, last, !:wb, !:wa),
+          Tree::Interpolation.new(str, mid, last, false, false),
           first.source_range.start_pos)
       end
 
@@ -614,6 +646,8 @@ RUBY
         e.force_division! if e
         end_pos = source_position
         assert_tok(:rparen)
+
+        @css_variable_warning.warn! if @css_variable_warning
         e || node(Sass::Script::Tree::ListLiteral.new([], nil), start_pos, end_pos)
       end
 
@@ -621,6 +655,8 @@ RUBY
         start_pos = source_position
         c = try_tok(:const)
         return string unless c
+
+        @css_variable_warning.warn! if @css_variable_warning
         node(Tree::Variable.new(*c.value), start_pos)
       end
 
@@ -629,7 +665,7 @@ RUBY
         return number unless first
         str = literal_node(first.value, first.source_range)
         return str unless try_tok(:string_interpolation)
-        mid = assert_expr :expr
+        mid = without_css_variable_warning {assert_expr :expr}
         assert_tok :end_interpolation
         last = assert_expr(:string)
         node(Tree::StringInterpolation.new(str, mid, last), first.source_range.start_pos)
@@ -647,6 +683,7 @@ RUBY
       def selector
         tok = try_tok(:selector)
         return literal unless tok
+        @css_variable_warning.warn! if @css_variable_warning
         node(tok.value, tok.source_range.start_pos)
       end
 
@@ -698,8 +735,14 @@ RUBY
       end
 
       def assert_done
-        return if @lexer.done?
-        @lexer.expected!(EXPR_NAMES[:default])
+        if @allow_extra_text
+          # If extra text is allowed, just rewind the lexer so that the
+          # StringScanner is pointing to the end of the parsed text.
+          @lexer.unpeek!
+        else
+          return if @lexer.done?
+          @lexer.expected!(EXPR_NAMES[:default])
+        end
       end
 
       # @overload node(value, source_range)
@@ -728,12 +771,27 @@ RUBY
             range(source_range_or_start_pos, end_pos)
           end
 
+        node.css_variable_warning = @css_variable_warning
         node.line = source_range.start_pos.line
         node.source_range = source_range
         node.filename = @options[:filename]
         node
       end
 
+      # Runs the given block without CSS variable warnings enabled.
+      #
+      # CSS warnings don't apply within interpolation, so this is used to
+      # disable them.
+      #
+      # @yield []
+      def without_css_variable_warning
+        old_css_variable_warning = @css_variable_warning
+        @css_variable_warning = nil
+        yield
+      ensure
+        @css_variable_warning = old_css_variable_warning
+      end
+
       # Checks a script node for any immediately-deprecated interpolations, and
       # emits warnings for them.
       #
