@@ -34,29 +34,31 @@ module Sass::Script::Value
     attr_accessor :original
 
     def self.precision
-      @precision ||= 5
+      Thread.current[:sass_numeric_precision] || Thread.main[:sass_numeric_precision] || 5
     end
 
     # Sets the number of digits of precision
     # For example, if this is `3`,
     # `3.1415926` will be printed as `3.142`.
+    # The numeric precision is stored as a thread local for thread safety reasons.
+    # To set for all threads, be sure to set the precision on the main thread.
     def self.precision=(digits)
-      @precision = digits.round
-      @precision_factor = 10.0**@precision
-      @epsilon = 1 / (@precision_factor * 10)
+      Thread.current[:sass_numeric_precision] = digits.round
+      Thread.current[:sass_numeric_precision_factor] = nil
+      Thread.current[:sass_numeric_epsilon] = nil
     end
 
     # the precision factor used in numeric output
     # it is derived from the `precision` method.
     def self.precision_factor
-      @precision_factor ||= 10.0**precision
+      Thread.current[:sass_numeric_precision_factor] ||= 10.0**precision
     end
 
     # Used in checking equality of floating point numbers. Any
     # numbers within an `epsilon` of each other are considered functionally equal.
     # The value for epsilon is one tenth of the current numeric precision.
     def self.epsilon
-      @epsilon ||= 1 / (precision_factor * 10)
+      Thread.current[:sass_numeric_epsilon] ||= 1 / (precision_factor * 10)
     end
 
     # Used so we don't allocate two new arrays for each new number.
@@ -71,6 +73,7 @@ module Sass::Script::Value
       super(value)
       @numerator_units = numerator_units
       @denominator_units = denominator_units
+      @options = nil
       normalize!
     end
 
@@ -469,11 +472,10 @@ module Sass::Script::Value
         sans_common_units(@numerator_units, @denominator_units)
 
       @denominator_units.each_with_index do |d, i|
-        if convertable?(d) && (u = @numerator_units.find(&method(:convertable?)))
-          @value /= conversion_factor(d, u)
-          @denominator_units.delete_at(i)
-          @numerator_units.delete_at(@numerator_units.index(u))
-        end
+        next unless convertable?(d) && (u = @numerator_units.find(&method(:convertable?)))
+        @value /= conversion_factor(d, u)
+        @denominator_units.delete_at(i)
+        @numerator_units.delete_at(@numerator_units.index(u))
       end
     end
 
@@ -508,8 +510,8 @@ module Sass::Script::Value
       },
       {
         'dpi'  => Rational(1),
-        'dpcm' => Rational(1, 2.54),
-        'dppx' => Rational(1, 96)
+        'dpcm' => Rational(254, 100),
+        'dppx' => Rational(96)
       }
     ]
 
