@@ -28,6 +28,8 @@ module Sass
         @line = line
         @offset = offset
         @strs = []
+        @expected = nil
+        @throw_error = false
       end
 
       # Parses an SCSS document.
@@ -509,7 +511,7 @@ module Sass
 
       def moz_document_function
         val = interp_uri || _interp_string(:url_prefix) ||
-          _interp_string(:domain) || function(!:allow_var) || interpolation
+          _interp_string(:domain) || function(false) || interpolation
         return unless val
         ss
         val
@@ -558,7 +560,7 @@ module Sass
       def supports_clause
         return unless tok(/supports\(/i)
         ss
-        supports = supports_condition
+        supports = import_supports_condition
         ss
         tok!(/\)/)
         supports
@@ -568,6 +570,10 @@ module Sass
         supports_negation || supports_operator || supports_interpolation
       end
 
+      def import_supports_condition
+        supports_condition || supports_declaration
+      end
+
       def supports_negation
         return unless tok(/not/i)
         ss
@@ -587,6 +593,13 @@ module Sass
         cond
       end
 
+      def supports_declaration
+          name = sass_script(:parse)
+          tok!(/:/); ss
+          value = sass_script(:parse)
+          Sass::Supports::Declaration.new(name, value)
+      end
+
       def supports_condition_in_parens
         interp = supports_interpolation
         return interp if interp
@@ -595,11 +608,9 @@ module Sass
           tok!(/\)/); ss
           cond
         else
-          name = sass_script(:parse)
-          tok!(/:/); ss
-          value = sass_script(:parse)
+          decl = supports_declaration
           tok!(/\)/); ss
-          Sass::Supports::Declaration.new(name, value)
+          decl
         end
       end
 
@@ -638,7 +649,7 @@ module Sass
         # are disallowed by the CSS spec,
         # but they're included here for compatibility
         # with some proprietary MS properties
-        str {ss if tok(/[\/,:.=]/)}
+        str {ss if tok(%r{[/,:.=]})}
       end
 
       def ruleset
@@ -764,7 +775,7 @@ module Sass
         value_start_pos = source_position
         value = nil
         error = catch_error do
-          value = value!
+          value = value!(name.first.is_a?(String) && name.first.start_with?("--"))
           if tok?(/\{/)
             # Properties that are ambiguous with selectors can't have additional
             # properties nested beneath them.
@@ -826,6 +837,9 @@ module Sass
           |
             (?!url\()
             [^"'/\#!;\{\}] # "
+          |
+            # interp_uri will handle most url() calls, but not ones that take strings
+            url\(#{W}(?=")
           |
             /(?![/*])
           |
@@ -857,7 +871,7 @@ module Sass
         tok!(/:/)
         ss
         value_start_pos = source_position
-        value = value!
+        value = value!(name.first.is_a?(String) && name.first.start_with?("--"))
         value_end_pos = source_position
         ss
         require_block = tok?(/\{/)
@@ -871,7 +885,7 @@ module Sass
         nested_properties! node
       end
 
-      def value!
+      def value!(css_variable = false)
         if tok?(/\{/)
           str = Sass::Script::Tree::Literal.new(Sass::Script::Value::String.new(""))
           str.line = source_position.line
@@ -891,10 +905,19 @@ module Sass
           str.source_range = range(start_pos)
           return str
         end
-        sass_script(:parse)
+
+        sass_script(:parse, css_variable)
       end
 
       def nested_properties!(node)
+        if node.name.first.is_a?(String) && node.name.first.start_with?("--")
+          Sass::Util.sass_warn(<<WARNING)
+DEPRECATION WARNING on line #{@line}#{" of #{@filename}" if @filename}:
+Sass 3.6 will change the way CSS variables are parsed. Instead of being parsed as
+normal properties, they will not allow any Sass-specific behavior other than \#{}.
+WARNING
+        end
+
         @expected = 'expression (e.g. 1px, bold) or "{"'
         block(node, :property)
       end
@@ -1027,8 +1050,7 @@ module Sass
         node
       end
 
-      @sass_script_parser = Class.new(Sass::Script::Parser)
-      @sass_script_parser.send(:include, ScriptParser)
+      @sass_script_parser = Sass::Script::Parser
 
       class << self
         # @private
@@ -1037,7 +1059,7 @@ module Sass
 
       def sass_script(*args)
         parser = self.class.sass_script_parser.new(@scanner, @line, @offset,
-                                                   :filename => @filename, :importer => @importer)
+          :filename => @filename, :importer => @importer, :allow_extra_text => true)
         result = parser.send(*args)
         unless @strs.empty?
           # Convert to CSS manually so that comments are ignored.
@@ -1102,7 +1124,7 @@ module Sass
 
         unless name
           # Display basic regexps as plain old strings
-          source = rx.source.gsub(/\\\//, '/')
+          source = rx.source.gsub(%r{\\/}, '/')
           string = rx.source.gsub(/\\(.)/, '\1')
           name = source == Regexp.escape(string) ? string.inspect : rx.inspect
         end
@@ -1133,14 +1155,20 @@ module Sass
         line = @line
         offset = @offset
         expected = @expected
+
+        logger = Sass::Logger::Delayed.install!
         if catch(:_sass_parser_error) {yield; false}
           @scanner.pos = pos
           @line = line
           @offset = offset
           @expected = expected
           {:pos => pos, :line => line, :expected => @expected, :block => block}
+        else
+          logger.flush
+          nil
         end
       ensure
+        logger.uninstall! if logger
         @throw_error = old_throw_error
       end
 
