@@ -457,6 +457,15 @@ foo {a: 1 + // flang }
 SCSS
   end
 
+  def test_static_hyphenated_unit
+    assert_equal <<CSS, render(<<SCSS)
+foo {
+  a: 0px; }
+CSS
+foo {a: 10px-10px }
+SCSS
+  end
+
   ## Nested Rules
 
   def test_nested_rules
@@ -1005,6 +1014,76 @@ bar {
 SASS
   end
 
+  def test_disallowed_function_names
+    assert_warning(<<WARNING) {render(<<SCSS)}
+DEPRECATION WARNING on line 1 of test_disallowed_function_names_inline.scss:
+Naming a function "calc" is disallowed and will be an error in future versions of Sass.
+This name conflicts with an existing CSS function with special parse rules.
+WARNING
+@function calc() {}
+SCSS
+
+    assert_warning(<<WARNING) {render(<<SCSS)}
+DEPRECATION WARNING on line 1 of test_disallowed_function_names_inline.scss:
+Naming a function "-my-calc" is disallowed and will be an error in future versions of Sass.
+This name conflicts with an existing CSS function with special parse rules.
+WARNING
+@function -my-calc() {}
+SCSS
+
+    assert_warning(<<WARNING) {render(<<SCSS)}
+DEPRECATION WARNING on line 1 of test_disallowed_function_names_inline.scss:
+Naming a function "element" is disallowed and will be an error in future versions of Sass.
+This name conflicts with an existing CSS function with special parse rules.
+WARNING
+@function element() {}
+SCSS
+
+    assert_warning(<<WARNING) {render(<<SCSS)}
+DEPRECATION WARNING on line 1 of test_disallowed_function_names_inline.scss:
+Naming a function "-my-element" is disallowed and will be an error in future versions of Sass.
+This name conflicts with an existing CSS function with special parse rules.
+WARNING
+@function -my-element() {}
+SCSS
+
+    assert_warning(<<WARNING) {render(<<SCSS)}
+DEPRECATION WARNING on line 1 of test_disallowed_function_names_inline.scss:
+Naming a function "expression" is disallowed and will be an error in future versions of Sass.
+This name conflicts with an existing CSS function with special parse rules.
+WARNING
+@function expression() {}
+SCSS
+
+    assert_warning(<<WARNING) {render(<<SCSS)}
+DEPRECATION WARNING on line 1 of test_disallowed_function_names_inline.scss:
+Naming a function "url" is disallowed and will be an error in future versions of Sass.
+This name conflicts with an existing CSS function with special parse rules.
+WARNING
+@function url() {}
+SCSS
+  end
+
+  def test_allowed_function_names
+    assert_no_warning {assert_equal(<<CSS, render(<<SCSS))}
+.a {
+  b: c; }
+CSS
+@function -my-expression() {@return c}
+
+.a {b: -my-expression()}
+SCSS
+
+    assert_no_warning {assert_equal(<<CSS, render(<<SCSS))}
+.a {
+  b: c; }
+CSS
+@function -my-url() {@return c}
+
+.a {b: -my-url()}
+SCSS
+  end
+
   ## Var Args
 
   def test_mixin_var_args
@@ -2226,14 +2305,14 @@ SCSS
 
   def test_supports_with_expressions
     assert_equal <<CSS, render(<<SCSS)
-@supports (feature1: val) and (feature2: val) or (not (feature23: val4)) {
+@supports ((feature1: val) and (feature2: val)) or (not (feature23: val4)) {
   foo {
     a: b; } }
 CSS
 $query: "(feature1: val)";
 $feature: feature2;
 $val: val;
-@supports \#{$query} and ($feature: $val) or (not ($feature + 3: $val + 4)) {
+@supports (\#{$query} and ($feature: $val)) or (not ($feature + 3: $val + 4)) {
   foo {a: b}
 }
 SCSS
@@ -3570,6 +3649,46 @@ SCSS
 
   # Regression
 
+  # Regression test for #2031.
+  def test_no_interpolation_warning_in_nested_selector
+    assert_no_warning {assert_equal(<<CSS, render(<<SCSS))}
+z a:b(n+1) {
+  x: y; }
+CSS
+z {
+  a:b(n+\#{1}) {
+    x: y;
+  }
+}
+SCSS
+  end
+
+  # Ensures that the fix for #2031 doesn't hide legitimate warnings.
+  def test_interpolation_warning_in_selector_like_property
+    assert_warning(<<WARNING) {assert_equal(<<CSS, render(<<SCSS))}
+DEPRECATION WARNING on line 2 of #{filename_for_test :scss}: \#{} interpolation near operators will be simplified
+in a future version of Sass. To preserve the current behavior, use quotes:
+
+  unquote("n+1")
+
+You can use the sass-convert command to automatically fix most cases.
+WARNING
+z {
+  a: b(n+1); }
+CSS
+z {
+  a:b(n+\#{1});
+}
+SCSS
+  end
+
+  def test_escape_in_selector
+    assert_equal(<<CSS, render(".\\!foo {a: b}"))
+.\\!foo {
+  a: b; }
+CSS
+  end
+
   def test_for_directive_with_float_bounds
     assert_equal(<<CSS, render(<<SCSS))
 .a {
@@ -3616,6 +3735,31 @@ CSS
 SCSS
   end
 
+  def test_parent_selector_in_and_out_of_function_pseudo_selector
+    # Regression test for https://github.com/sass/sass/issues/1464#issuecomment-70352288
+    assert_equal(<<CSS, render(<<SCSS))
+.a:not(.a-b) {
+  x: y; }
+CSS
+.a {
+  &:not(&-b) {
+    x: y;
+  }
+}
+SCSS
+
+    assert_equal(<<CSS, render(<<SCSS))
+.a:nth-child(2n of .a-b) {
+  x: y; }
+CSS
+.a {
+  &:nth-child(2n of &-b) {
+    x: y;
+  }
+}
+SCSS
+  end
+
   def test_attribute_selector_in_selector_pseudoclass
     # Even though this is plain CSS, it only failed when given to the SCSS
     # parser.
@@ -3823,6 +3967,22 @@ CSS
 SCSS
   end
 
+  def test_keyword_arg_scope
+    assert_equal <<CSS, render(<<SCSS)
+.mixed {
+  arg1: default;
+  arg2: non-default; }
+CSS
+$arg1: default;
+$arg2: default;
+@mixin a-mixin($arg1: $arg1, $arg2: $arg2) {
+  arg1: $arg1;
+  arg2: $arg2;
+}
+.mixed { @include a-mixin($arg2: non-default); }
+SCSS
+  end
+
   def test_passing_required_args_as_a_keyword_arg
     assert_equal <<CSS, render(<<SCSS)
 .mixed {
@@ -3998,4 +4158,33 @@ CSS
 }
 SCSS
   end
+
+  def test_import_with_supports_clause_interp
+    assert_equal(<<CSS, render(<<'SASS', :style => :compressed))
+@import url("fallback-layout.css") supports(not (display: flex))
+CSS
+$display-type: flex;
+@import url("fallback-layout.css") supports(not (display: #{$display-type}));
+SASS
+  end
+
+  def test_import_with_supports_clause
+    assert_equal(<<CSS, render(<<SASS, :style => :compressed))
+@import url("fallback-layout.css") supports(not (display: flex));.foo{bar:baz}
+CSS
+@import url("fallback-layout.css") supports(not (display: flex));
+.foo { bar: baz; }
+SASS
+  end
+
+  def test_crlf
+    # Attempt to reproduce https://github.com/sass/sass/issues/1985
+    assert_equal(<<CSS, render(<<SCSS))
+p {
+  margin: 0; }
+CSS
+p {\r\n   margin: 0;\r\n}
+SCSS
+  end
+
 end
